@@ -6,4 +6,5 @@ GNU General Public License v3.0 only
 SPDX-License-Identifier: GPL-3.0-only
 """
 
-from .event_manager import SubscribedEvent, EventPayload
+from .event_manager import SubscribedEvent, EventPayload, \
+    EventWithNotify, EventNotificationName, send_to_outbound_queue
